# Copyright (c) 2017 Turbopay Holdings Ltd

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE

import base64
from datetime import datetime

from atomicpay.mpesa import api, routes


class Lipa(object):
    """Lipa Na M-Pesa online payment using STK Push

    Attributes:
        client: The mpesa client contains the auth credentials and live status
    Reference:
        https://developer.safaricom.co.ke/lipa-na-m-pesa-online/apis
    """

    def __init__(self, client):
        self.client = client

    def pay(self, phone, amount, description, reference, **kwargs):

        time = self.timestamp()

        passwd = self.generate_password(
            self.client.configs['LipaShortcode'],
            self.client.configs['LipaPassKey'],
            time
        )

        callback = kwargs.pop("callback", None)

        data = {
            "AccountReference": reference,
            "Amount": int(amount),
            "BusinessShortCode": int(self.client.configs['LipaShortcode']),
            "CallBackURL": kwargs.get('callback_url') or self.client.configs['URLS']['LIPA']['CallBackURL'],
            "PartyA": int(phone),
            "PartyB": int(self.client.configs['LipaShortcode']),
            "Password": passwd,
            "PhoneNumber": int(phone),
            "Timestamp": time,
            "TransactionDesc": description,
            "TransactionType": "CustomerPayBillOnline",
        }

        return api.call(
            route=routes.LNMOnlinePayment,
            data=data,
            access_token=self.client.auth.access_token,
            live=self.client.live,
            callback=callback
        )

    def query(self, checkout_id, **kwargs):
        time = self.timestamp()

        passwd = self.generate_password(
            self.client.configs['LipaShortcode'],
            self.client.configs['LipaPassKey'],
            time
        )

        callback = kwargs.pop("callback", None)

        data = {
            "BusinessShortCode": int(self.client.configs['LipaShortcode']),
            "CheckoutRequestID": checkout_id,
            "Password": passwd,
            "Timestamp": time
        }

        return api.call(
            route=routes.LNMQueryRequest,
            data=data,
            access_token=self.client.auth.access_token,
            live=self.client.live,
            callback=callback
        )

    @staticmethod
    def generate_password(shortcode, passkey, timestamp):
        """
        The password for encrypting the request.\n
        Generated by base64 encoding (BusinessShortcode, Passkey and Timestamp).
        """
        r = str(shortcode) + str(passkey) + str(timestamp)
        string_bytes = bytes(r, encoding='utf-8')

        encoded = base64.b64encode(string_bytes)

        return encoded.decode("utf-8")

    @staticmethod
    def timestamp():
        """Timestamp in the format yyyymmddhhiiss."""
        return datetime.now().strftime("%Y%m%d%H%M%S")
